@@ -97,7 +97,7 @@ class FMIndex(object):
         if self._rawmode:
             doc = []
             for substr in self._substr:
-                doc = doc + substr
+                doc += substr
         else:
             doc = "".join(self._substr)
         return doc
@@ -131,8 +131,14 @@ class FMIndex(object):
         return struct.pack('<%dH' % len(codes), *codes).decode('utf_16_le')
 
     def build(self, ddic, maxChar=65535):
+        import time
+        time1 = time.time()
         doc = self._join()
+        time2 = time.time()
+        #print("@@debug: join: %f" % (time2 - time1))
         sa = bwt.BWT(doc, rawmode=self._rawmode)
+        time3 = time.time()
+        #print("@@debug: bwt: %f" % (time3 - time2))
         s = sa.get()
         self._ssize = len(s)
         self._head = sa.head()
@@ -140,11 +146,19 @@ class FMIndex(object):
         self._sv.set_max_char_code(maxChar)
         self._sv.build(s)
         size = self.size()
+        time4 = time.time()
+        #print("@@debug: build wavelet matrix: %f" % (time4 - time3))
         for c in _range(maxChar):
             self._rlt[c] = self._sv.rank_less_than(size, c)
+        time5 = time.time()
+        #print("@@debug: calc rank: %f" % (time5 - time4))
         self._rlt[maxChar] = 0;
         self._ddic = int(ddic)
+        import cProfile
+        cProfile.runctx("self._buildDictionaries()", globals(), locals())
         self._buildDictionaries()
+        time6 = time.time()
+        #print("@@debug: build dict: %f" % (time6 - time5))
         self._build = True
 
     def _buildDictionaries(self):
